# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for model_card."""

import json
import os

from absl.testing import absltest
import jsonschema

from model_card_toolkit.model_card import Dataset
from model_card_toolkit.model_card import ModelCard
from model_card_toolkit.model_card import Owner

_SCHEMA_DIR = os.path.join(os.path.dirname(__file__), 'schema')
_SCHEMA_FILE = 'model_card.schema.json'


class ModelCardTest(absltest.TestCase):

  def _validate_schema(self, model_card: ModelCard) -> None:
    """Validates the model_card against the json schema.

    Args:
      model_card: The model card data object.

    Raises:
       jsonschema.ValidationError: when the given model_card is
         invalid w.r.t. the schema.
    """
    path = model_card.schema_version if model_card.schema_version else '0.0.1'
    schema_file = os.path.join(_SCHEMA_DIR, 'v' + path, _SCHEMA_FILE)
    with open(schema_file) as json_file:
      schema = json.loads(json_file.read())
    jsonschema.validate(model_card.to_dict(), schema)

  def test_empty_model_card_is_valid_json(self):
    model_card = ModelCard()
    self._validate_schema(model_card)

  def _fill_model_details(self, model_card: ModelCard) -> None:
    """Fills the model_details of the model card."""
    model_details = model_card.model_details
    model_details.name = 'my model'
    model_details.owners = [
        Owner(name='foo', contact='foo@xyz.com'),
        {
            'name': 'bar',
            'contact': 'bar@xyz.com'
        },
    ]
    model_details.version.name = '0.1'
    model_details.version.date = '2020-01-01'
    model_details.license = 'Apache 2.0'
    model_details.references = ['https://my_model.xyz.com']
    model_details.citation = 'https://doi.org/foo/bar'

  def test_model_card_with_model_details_is_valid_json(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_model_details(model_card)
    self._validate_schema(model_card)

  def _fill_model_parameters(self, model_card: ModelCard) -> None:
    """Fills the model_parameters of the model card."""
    model_parameters = model_card.model_parameters
    model_parameters.model_architecture = 'knn'
    model_parameters.data.train.name = 'train_split'
    model_parameters.data.train.link = 'path/to/train'
    model_parameters.data.train.sensitive = False
    model_parameters.data.train.graphics.collection.append({
        'name': 'image1',
        'image': 'rawbytes'
    })
    model_parameters.data.eval = Dataset(name='eval_split', link='path/to/eval')

  def test_model_card_with_model_parameters_is_valid_json(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_model_parameters(model_card)
    self._validate_schema(model_card)

  def _fill_quantitative_analysis(self, model_card: ModelCard) -> None:
    """Fills the quantitative_analysis section of the metadata card."""
    model_card.quantitative_analysis.graphics.collection.append({
        'name': 'image1',
        'image': 'rawbytes'
    })
    model_card.quantitative_analysis.performance_metrics.append({
        'type': 'log_loss',
        'value': 0.2,
    })

  def test_model_card_with_quantitative_analysis_is_valid_json(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_quantitative_analysis(model_card)
    self._validate_schema(model_card)

  def _fill_considerations(self, model_card: ModelCard) -> None:
    """Fills the considerations section of the metadata card."""
    model_card.considerations.users = ['foo', 'bar']
    model_card.considerations.use_cases.append('use case 1')
    model_card.considerations.limitations.append('a limitation')
    model_card.considerations.tradeoffs.append('tradeoff 1')
    model_card.considerations.ethical_considerations.append({
        'name': 'risk1',
        'mitigation_strategy': 'a solution'
    })

  def test_model_card_with_considerations_is_valid_json(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_considerations(model_card)
    self._validate_schema(model_card)

  def test_full_filled_model_card_is_valid_json(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_model_details(model_card)
    self._fill_model_parameters(model_card)
    self._fill_quantitative_analysis(model_card)
    self._fill_considerations(model_card)
    self._validate_schema(model_card)

  def test_default_value_not_shared_among_model_cards(self):
    model_card = ModelCard()
    model_card.schema_version = '0.0.1'
    self._fill_model_details(model_card)
    self._fill_model_parameters(model_card)
    self._fill_quantitative_analysis(model_card)
    self._fill_considerations(model_card)
    other_model_card = ModelCard()
    self.assertNotEqual(other_model_card, model_card)
    self.assertEqual(other_model_card, ModelCard())


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Model Cards Toolkit.

The model cards toolkit (MCT) provides a set of utilities to help users
generate Model Cards from trained models within ML pipelines.
"""

import json
import os
import os.path
import shutil
import tempfile
from typing import Any, Dict, Optional, Text

from absl import logging
import jinja2
import jsonschema

from model_card_toolkit import model_card as model_card_module
from model_card_toolkit.utils import graphics
from model_card_toolkit.utils import tfx_util

import semantic_version

from ml_metadata.metadata_store import metadata_store

# Constants about versioned JSON schema files for Model Card.
_SCHEMA_DIR = os.path.join(os.path.dirname(__file__), 'schema')
_SCHEMA_FILE_NAME = 'model_card.schema.json'
# Constants about provided UI templates.
_UI_TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), 'template')
_DEFAULT_UI_TEMPLATE_FILE = 'html/default_template.html.jinja'
# Constants about Model Cards Toolkit Assets (MCTA).
_MCTA_JSON_FILE = 'data/model_card.json'
_MCTA_TEMPLATE_DIR = 'template'
_MCTA_RESOURCE_DIR = 'resources/plots'
# Constants about the final generated model cards.
_DEFAULT_MODEL_CARD_FILE_NAME = 'model_card.html'
_MODEL_CARDS_DIR = 'model_cards/'


class ModelCardToolkit():
  """Model Cards Toolkit (MCT) provides utilities to generate a ModelCard.

  Given a model, a ModelCardToolkit finds related metadata and lineage from a
  MLMD instance and generates a ModelCard about the model in various output
  formats (e.g., HTML). The ModelCardToolkit includes a list of APIs designed
  for a human-in-the-loop process to elaborate the ModelCard. It organizes the
  ModelCard assets (e.g., structured data, plots, and UI templates) in a user
  specified directory, and updates them incrementally via the APIs.
  """

  def __init__(self,
               output_dir: Optional[Text] = None,
               mlmd_store: Optional[metadata_store.MetadataStore] = None,
               model_uri: Optional[Text] = None):
    """Initializes the ModelCardToolkit.

    Args:
      output_dir: The MCT assets path where the json file and templates are
        written to. If not given, a temp directory is used.
      mlmd_store: A ml-metadata MetadataStore to retrieve metadata and lineage
        information about the model stored at `model_uri`. If given, a set of
        model card properties can be auto-populated from the `mlmd_store`.
      model_uri: The path to the trained model to generate model cards.

    Raises:
      ValueError: If `mlmd_store` is given and the `model_uri` cannot be
        resolved as a model artifact in the metadata store.
    """
    self.output_dir = output_dir or tempfile.mkdtemp()
    self._mcta_json_file = os.path.join(self.output_dir, _MCTA_JSON_FILE)
    self._mcta_template_dir = os.path.join(self.output_dir, _MCTA_TEMPLATE_DIR)
    self._model_cards_dir = os.path.join(self.output_dir, _MODEL_CARDS_DIR)
    self._store = mlmd_store
    if self._store:
      if not model_uri:
        raise ValueError('If `mlmd_store` is set, `model_uri` should be set.')
      models = self._store.get_artifacts_by_uri(model_uri)
      if not models:
        raise ValueError(f'"{model_uri}" cannot be found in the `mlmd_store`.')
      if len(models) > 1:
        logging.info(
            '%d artifacts are found with the `model_uri`="%s". '
            'The last one is used.', len(models), model_uri)
      self._artifact_with_model_uri = models[-1]

  def _write_file(self, path: Text, content: Text) -> None:
    """Write content to the path."""
    os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, 'w+') as f:
      f.write(content)

  def _read_file(self, path: Text) -> Text:
    """Read content from a path."""
    with open(path, 'r') as f:
      return f.read()

  def scaffold_assets(self) -> model_card_module.ModelCard:
    """Generates the model cards tookit assets.

    Model cards assets include the model card json file and customizable model
    card UI templates.

    An assets directory is created if one does not already exist.

    If the MCT is initialized with a `mlmd_store`, it further auto-populates
    the model cards properties as well as generating related plots such as model
    performance and data distributions.

    Returns:
      A ModelCard representing the given model.
    """
    model_card = model_card_module.ModelCard()
    if self._store:
      model_card = tfx_util.generate_model_card_for_model(
          self._store, self._artifact_with_model_uri.id)
      metrics_artifacts = tfx_util.get_metrics_artifacts_for_model(
          self._store, self._artifact_with_model_uri.id)
      stats_artifacts = tfx_util.get_stats_artifacts_for_model(
          self._store, self._artifact_with_model_uri.id)

      for metrics_artifact in metrics_artifacts:
        eval_result = tfx_util.read_metrics_eval_result(metrics_artifact.uri)
        if eval_result is not None:
          graphics.annotate_eval_result_plots(model_card, eval_result)

      for stats_artifact in stats_artifacts:
        train_stats = tfx_util.read_stats_proto(stats_artifact.uri, 'train')
        eval_stats = tfx_util.read_stats_proto(stats_artifact.uri, 'eval')
        graphics.annotate_dataset_feature_statistics_plots(
            model_card, train_stats, eval_stats)

    # Write JSON file.
    self._write_file(self._mcta_json_file, model_card.to_json())
    # Write UI template files.
    shutil.copytree(_UI_TEMPLATE_DIR, self._mcta_template_dir)
    return model_card

  def update_model_card_json(self,
                             model_card: model_card_module.ModelCard) -> None:
    """Validates the model card and updates the JSON file in MCT assets.

    If model_card.schema_version is not provided, it will assign the latest
    schema version to the `model_card`, and validate it.

    Args:
      model_card: The updated model card that users want to write back.

    Raises:
       Error: when the given model_card is invalid w.r.t. the schema.
    """
    if not model_card.schema_version:
      sub_directories = [f for f in os.scandir(_SCHEMA_DIR) if f.is_dir()]
      latest_schema_version = max(
          sub_directories, key=lambda f: semantic_version.Version(f.name[1:]))
      model_card.schema_version = latest_schema_version.name[1:]
    # Validate the updated model_card first.
    schema = self._find_model_card_schema(model_card.schema_version)
    jsonschema.validate(model_card.to_dict(), schema)
    # Write the updated JSON to the file.
    self._write_file(self._mcta_json_file, model_card.to_json())

  def export_format(self,
                    template_path: Text = None,
                    output_file=_DEFAULT_MODEL_CARD_FILE_NAME) -> Text:
    """Generates a model card based on the MCT assets.

    Args:
      template_path: The file path of the UI template. If not provided, the
        default UI template will be used.
      output_file: The file name of the generated model card. If not provided,
        the default 'model_card.html' will be used. If the file already exists,
        then it will be overwritten.

    Returns:
      The model card UI.
    """
    if not template_path:
      template_path = os.path.join(self._mcta_template_dir,
                                   _DEFAULT_UI_TEMPLATE_FILE)

    template_dir = os.path.dirname(template_path)
    template_file = os.path.basename(template_path)

    # Read JSON file.
    model_card = json.loads(self._read_file(self._mcta_json_file))

    # Generate Model Card.
    jinja_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(template_dir),
        trim_blocks=True,
        lstrip_blocks=True,
        autoescape=True,
        auto_reload=True,
        cache_size=0)
    template = jinja_env.get_template(template_file)
    # TODO(b/154990170) Think about how to adjust the img inside template.
    model_card_file_content = template.render(model_card=model_card)

    # Write the model card file.
    mode_card_file_path = os.path.join(self._model_cards_dir, output_file)
    self._write_file(mode_card_file_path, model_card_file_content)

    return model_card_file_content

  def save_mlmd(self) -> None:
    """Saves the model card of the model artifact with `model_uri` to MLMD."""
    pass

  def _find_model_card_schema(self, version: Text) -> Dict[Text, Any]:
    """Finds the model card JSON schema of a particular version.

    A model card is created w.r.t. to ModelCard json schema. The MCT contains
    a list of known Model Card JSON schemas. The util looks for the json schema
    file at a particular version and returns for validation.

    Args:
      version: The version of the schema.

    Returns:
      Json schema.

    Raises:
      ValueError if cannot find the expect schema given the version.
    """
    sub_directories = [f for f in os.scandir(_SCHEMA_DIR) if f.is_dir()]
    # Remove the first charactor 'v' from path when comparing.
    matching_dir = [f for f in sub_directories if str(f.name[1:]) == version]
    if not matching_dir:
      raise ValueError(
          'Cannot find schema version that matches the version of the given '
          'model card. Found Versions: {}. Given Version: {}'.format(
              str([str(f.name[1:]) for f in sub_directories]), version))
    schema_file = os.path.join(str(matching_dir[0].path), _SCHEMA_FILE_NAME)
    with open(schema_file) as json_file:
      schema = json.loads(json_file.read())
    return schema



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for the TFX-OSS pipeline utilities."""

import os
import uuid

from absl.testing import absltest

from model_card_toolkit.utils import tfx_util
from model_card_toolkit.utils.testdata import testdata_utils
from ml_metadata.metadata_store import metadata_store
from ml_metadata.proto import metadata_store_pb2


class TfxUtilsTest(absltest.TestCase):

  def setUp(self):
    super(TfxUtilsTest, self).setUp()
    self.tmp_db_path = os.path.join(absltest.get_default_test_tmpdir(),
                                    f'test_mlmd_{uuid.uuid4()}.db')

  def _get_empty_metadata_store(self):
    """Returns an empty in memory mlmd store."""
    empty_db_config = metadata_store_pb2.ConnectionConfig()
    empty_db_config.fake_database.SetInParent()
    return metadata_store.MetadataStore(empty_db_config)

  def test_get_metrics_artifacts_for_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    got_metrics = tfx_util.get_metrics_artifacts_for_model(
        store, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)
    got_metrics_ids = [a.id for a in got_metrics]
    self.assertCountEqual(got_metrics_ids,
                          testdata_utils.TFX_0_21_METRICS_ARTIFACT_IDS)

  def test_get_metrics_artifacts_for_model_model_with_model_not_found(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'model_id cannot be found'):
      model = metadata_store_pb2.Artifact()
      tfx_util.get_metrics_artifacts_for_model(store, model.id)

  def test_get_metrics_artifacts_for_model_with_invalid_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'not an instance of Model'):
      tfx_util.get_metrics_artifacts_for_model(
          store, testdata_utils.TFX_0_21_MODEL_DATASET_ID)

  def test_get_metrics_artifacts_for_model_with_invalid_db(self):
    empty_db = self._get_empty_metadata_store()
    with self.assertRaisesRegex(ValueError, '`store` is invalid'):
      tfx_util.get_metrics_artifacts_for_model(
          empty_db, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)

  def test_get_stats_artifacts_for_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    got_stats = tfx_util.get_stats_artifacts_for_model(
        store, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)
    got_stats_ids = [a.id for a in got_stats]
    self.assertCountEqual(got_stats_ids,
                          [testdata_utils.TFX_0_21_STATS_ARTIFACT_ID])

  def test_get_stats_artifacts_for_model_with_model_not_found(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'model_id cannot be found'):
      model = metadata_store_pb2.Artifact()
      tfx_util.get_stats_artifacts_for_model(store, model.id)

  def test_get_stats_artifacts_for_model_with_invalid_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'not an instance of Model'):
      tfx_util.get_stats_artifacts_for_model(
          store, testdata_utils.TFX_0_21_MODEL_DATASET_ID)

  def test_get_stats_artifacts_for_model_with_invalid_db(self):
    empty_db = self._get_empty_metadata_store()
    with self.assertRaisesRegex(ValueError, '`store` is invalid'):
      tfx_util.get_stats_artifacts_for_model(
          empty_db, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)

  def test_generate_model_card_for_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    model_card = tfx_util.generate_model_card_for_model(
        store, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)
    trainers = store.get_executions_by_id([testdata_utils.TFX_0_21_TRAINER_ID])
    self.assertNotEmpty(trainers)
    model_details = model_card.model_details
    self.assertEqual(model_details.name,
                     trainers[-1].properties['module_file'].string_value)
    self.assertEqual(model_details.version.name,
                     trainers[-1].properties['checksum_md5'].string_value)
    self.assertIn(trainers[-1].properties['pipeline_name'].string_value,
                  model_details.references)

    datasets = store.get_artifacts_by_id(
        [testdata_utils.TFX_0_21_MODEL_DATASET_ID])
    self.assertNotEmpty(datasets)
    model_params = model_card.model_parameters
    self.assertStartsWith(model_params.data.train.name, datasets[-1].uri)
    self.assertStartsWith(model_params.data.eval.name, datasets[-1].uri)

  def test_generate_model_card_for_model_with_model_not_found(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'model_id cannot be found'):
      model = metadata_store_pb2.Artifact()
      tfx_util.generate_model_card_for_model(store, model.id)

  def test_generate_model_card_for_model_with_invalid_model(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(ValueError, 'not an instance of Model'):
      tfx_util.generate_model_card_for_model(
          store, testdata_utils.TFX_0_21_MODEL_DATASET_ID)

  def test_generate_model_card_for_model_with_invalid_db(self):
    empty_db = self._get_empty_metadata_store()
    with self.assertRaisesRegex(ValueError, '`store` is invalid'):
      tfx_util.generate_model_card_for_model(
          empty_db, testdata_utils.TFX_0_21_MODEL_ARTIFACT_ID)

  def test_read_stats_proto(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    stats = store.get_artifacts_by_id(
        [testdata_utils.TFX_0_21_STATS_ARTIFACT_ID])
    self.assertLen(stats, 1)
    train_stats = tfx_util.read_stats_proto(stats[-1].uri, 'train')
    self.assertIsNotNone(train_stats)
    eval_stats = tfx_util.read_stats_proto(stats[-1].uri, 'eval')
    self.assertIsNotNone(eval_stats)

  def test_read_stats_proto_with_invalid_split(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    stats = store.get_artifacts_by_id(
        [testdata_utils.TFX_0_21_STATS_ARTIFACT_ID])
    self.assertLen(stats, 1)
    actual_stats = tfx_util.read_stats_proto(stats[-1].uri, 'invalid_split')
    self.assertIsNone(actual_stats)

  def test_read_stats_proto_with_invalid_uri(self):
    self.assertIsNone(tfx_util.read_stats_proto('/does/not/exist/', 'train'))
    self.assertIsNone(tfx_util.read_stats_proto('/does/not/exist/', 'eval'))

  def test_read_metrics_eval_result(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    metrics = store.get_artifacts_by_id(
        testdata_utils.TFX_0_21_METRICS_ARTIFACT_IDS)
    eval_result = tfx_util.read_metrics_eval_result(metrics[-1].uri)
    self.assertIsNotNone(eval_result)

  def test_read_metrics_eval_result_with_invalid_uri(self):
    self.assertIsNone(tfx_util.read_metrics_eval_result('/does/not/exist/'))


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Model Card Data Class.

The Model Card (MC) is the document designed for transparent reporting of AI
model provenance, usage, and ethics-informed evaluation. The model card can be
presented by different formats (e.g. HTML, PDF, Markdown). The properties of
the Model Card (MC) are defined by a json schema. The ModelCard class in the
ModelCardsToolkit serves as an API to read and write MC properties by the users.
"""

import json
from typing import Any, Dict, List, Optional, Text, Union

import attr


@attr.s(auto_attribs=True)
class Version():
  """The information about verions of a model."""
  # The name of the version.
  name: Optional[Text] = None
  # The date the version was released.
  date: Optional[Text] = None
  # The changes from the previous version.
  diff: Optional[Text] = None


@attr.s(auto_attribs=True)
class Owner():
  """The information about owners of a model."""
  # The name of the owner.
  name: Optional[Text] = None
  # The contact information of the owner.
  contact: Optional[Text] = None


@attr.s(auto_attribs=True)
class ModelDetails():
  """Metadata about the model."""
  # The name of the model.
  name: Optional[Text] = None
  # A description of the model card.
  overview: Optional[Text] = None
  # The individuals or teams who own the model.
  owners: List[Owner] = attr.Factory(list)
  # The version of the model.
  version: Version = attr.Factory(Version)
  # The model's license for use.
  license: Optional[Text] = None
  # Links providing more information about the model.
  references: List[Text] = attr.Factory(list)
  # How to reference this model card.
  citation: Optional[Text] = None


@attr.s(auto_attribs=True)
class Graphic():
  """A named inline plot."""
  # The name of the graphic.
  name: Text
  # The image string encoded as a base64 string.
  image: Text


@attr.s(auto_attribs=True)
class Graphics():
  """A collection of graphics."""
  # A description of this collection of graphics.
  description: Optional[Text] = None
  # A collection of graphics.
  collection: List[Graphic] = attr.Factory(list)


@attr.s(auto_attribs=True)
class Dataset():
  """The information about a dataset used to generate a model."""
  # The name of the dataset.
  name: Optional[Text] = None
  # The contact information of the owner
  link: Optional[Text] = None
  # Does this dataset contain human or other sensitive data?
  sensitive: Optional[bool] = None
  # Visualizations of the dataset.
  graphics: Graphics = attr.Factory(Graphics)


@attr.s(auto_attribs=True)
class Data():
  """The related datasets used to train and evaluate the model."""
  # The training dataset
  train: Dataset = attr.Factory(Dataset)
  # The evaluation dataset
  eval: Dataset = attr.Factory(Dataset)


@attr.s(auto_attribs=True)
class ModelParameters():
  """Parameters for construction of the model."""
  # The architecture of the model.
  model_architecture: Optional[Text] = None
  # The datasets used to train and evaluate the model.
  data: Data = attr.Factory(Data)


@attr.s(auto_attribs=True)
class PerformanceMetric():
  """The details of the performance metric."""
  # The type of performance metric.
  type: Text
  # The value of the performance metric.
  value: Union[int, float, Text]
  # The decision threshold the metric was computed on.
  threshold: Optional[float] = None


@attr.s(auto_attribs=True)
class QuantitativeAnalysis():
  """The quantitative analysis of a model."""
  # The model performance metrics being reported.
  performance_metrics: List[PerformanceMetric] = attr.Factory(list)
  # Visualizations of model performance.
  graphics: Graphics = attr.Factory(Graphics)


@attr.s(auto_attribs=True)
class Risk():
  """The information about risks when using the model."""
  # The name of the risk.
  name: Text
  # Strategy used to address this risk.
  mitigation_strategy: Text


@attr.s(auto_attribs=True)
class Considerations():
  """Considerations related to model construction, training, and application."""
  # Who are the intended users of the model?
  users: List[Text] = attr.Factory(list)
  # What are the intended use cases of the model.
  use_cases: List[Text] = attr.Factory(list)
  # What are the known technical limitations of the model.
  limitations: List[Text] = attr.Factory(list)
  # What are the known tradeoffs in accuracy/performance of the model
  tradeoffs: List[Text] = attr.Factory(list)
  # What are the ethical risks involved in the application of this model.
  ethical_considerations: List[Risk] = attr.Factory(list)


@attr.s(auto_attribs=True)
class ModelCard(object):
  """A class that represents assets of ModelCards created by the MCT."""
  # The json schema version of the ModelCard
  schema_version: Optional[Text] = None
  # Descriptive metadata for the model.
  model_details: ModelDetails = attr.Factory(ModelDetails)
  # Parameters used when generating the model.
  model_parameters: ModelParameters = attr.Factory(ModelParameters)
  # The quantitative analysis of the ModelCard
  quantitative_analysis: QuantitativeAnalysis = attr.Factory(
      QuantitativeAnalysis)
  # The considerations related to model construction, training, and application.
  considerations: Considerations = attr.Factory(Considerations)

  def to_dict(self) -> Dict[Text, Any]:
    # ignore None properties recusively to allow missing values.
    return attr.asdict(self, filter=lambda attr, value: value)

  def to_json(self) -> Text:
    return json.dumps(self.to_dict(), indent=2)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for reading metadata from MLMD instances in TFX-OSS pipelines."""

import enum
import os
from typing import Iterable, List, Optional, Text, Union

from absl import logging
import attr
from model_card_toolkit.model_card import ModelCard
import tensorflow as tf
import tensorflow_model_analysis as tfma

from ml_metadata.metadata_store import metadata_store
from ml_metadata.proto import metadata_store_pb2
from tensorflow_metadata.proto.v0 import statistics_pb2

# A list of artifact type names used by TFX 0.21 and later versions.
_TFX_DATASET_TYPE = 'Examples'
_TFX_STATS_TYPE = 'ExampleStatistics'
_TFX_MODEL_TYPE = 'Model'
_TFX_METRICS_TYPE = 'ModelEvaluation'
_TFX_TRAINER_TYPE = 'tfx.components.trainer.component.Trainer'


@attr.s(auto_attribs=True)
class _PipelineTypes(object):
  """A registry of required MLMD types about a TFX pipeline."""
  # a list of required artifact types
  dataset_type: metadata_store_pb2.ArtifactType
  stats_type: metadata_store_pb2.ArtifactType
  model_type: metadata_store_pb2.ArtifactType
  metrics_type: metadata_store_pb2.ArtifactType
  # a list of required execution types
  trainer_type: metadata_store_pb2.ExecutionType


def _get_tfx_pipeline_types(
    store: metadata_store.MetadataStore) -> _PipelineTypes:
  """Retrieves the registered types in the given `store`.

  Args:
    store: A ml-metadata MetadataStore to retrieve ArtifactTypes from.

  Returns:
    A instance of _PipelineTypes containing store pipeline types.

  Raises:
    ValueError: If the `store` does not have MCT related types and is not
      considered a valid TFX store.
  """
  artifact_types = {atype.name: atype for atype in store.get_artifact_types()}
  expected_artifact_types = {
      _TFX_DATASET_TYPE, _TFX_STATS_TYPE, _TFX_MODEL_TYPE, _TFX_METRICS_TYPE
  }
  missing_types = expected_artifact_types.difference(artifact_types.keys())
  if missing_types:
    raise ValueError(
        f'Given `store` is invalid: missing ArtifactTypes: {missing_types}.'
    )
  execution_types = {etype.name: etype for etype in store.get_execution_types()}
  expected_execution_types = {_TFX_TRAINER_TYPE}
  missing_types = expected_execution_types.difference(execution_types.keys())
  if missing_types:
    raise ValueError(
        f'Given `store` is invalid: missing ExecutionTypes: {missing_types}.')
  return _PipelineTypes(
      dataset_type=artifact_types[_TFX_DATASET_TYPE],
      stats_type=artifact_types[_TFX_STATS_TYPE],
      model_type=artifact_types[_TFX_MODEL_TYPE],
      metrics_type=artifact_types[_TFX_METRICS_TYPE],
      trainer_type=execution_types[_TFX_TRAINER_TYPE])


def _validate_model_id(store: metadata_store.MetadataStore,
                       model_type: metadata_store_pb2.ArtifactType,
                       model_id: int) -> None:
  """Validates the given `model_id` against the `store`.

  Args:
    store: A ml-metadata MetadataStore to be validated.
    model_type: The Model ArtifactType in the `store`.
    model_id: The id for the model artifact in the `store`.

  Raises:
    ValueError: If the `model_id` cannot be resolved as a Model artifact in the
      given `store`.
  """
  model_artifacts = store.get_artifacts_by_id([model_id])
  if not model_artifacts:
    raise ValueError(f'Input model_id cannot be found: {model_id}.')
  model = model_artifacts[0]
  if model.type_id != model_type.id:
    raise ValueError(
        f'Found artifact with `model_id` is not an instance of Model: {model}.')


@enum.unique
class _Direction(enum.Enum):
  """An enum of directions when traversing MLMD lineage."""
  ANCESTOR = 1
  SUCCESSOR = 2


def _get_one_hop_artifacts(
    store: metadata_store.MetadataStore,
    artifact_ids: Iterable[int],
    direction: _Direction,
    filter_type: Optional[metadata_store_pb2.ArtifactType] = None
) -> List[metadata_store_pb2.Artifact]:
  """Gets a list of artifacts within 1-hop neighborhood of the `artifact_ids`.

  Args:
    store: A ml-metadata MetadataStore to look for neighborhood artifacts.
    artifact_ids: The artifacts' ids in the `store`.
    direction: A direction to specify whether returning ancestors or successors.
    filter_type: An optional type filter of the returned artifacts, if given
      then only artifacts of that type is returned.

  Returns:
    A list of qualified artifacts within 1-hop neighborhood in the `store`.
  """
  traverse_events = {}
  if direction == _Direction.ANCESTOR:
    traverse_events['execution'] = metadata_store_pb2.Event.OUTPUT
    traverse_events['artifact'] = metadata_store_pb2.Event.INPUT
  elif direction == _Direction.SUCCESSOR:
    traverse_events['execution'] = metadata_store_pb2.Event.INPUT
    traverse_events['artifact'] = metadata_store_pb2.Event.OUTPUT
  executions_ids = set(
      event.execution_id
      for event in store.get_events_by_artifact_ids(artifact_ids)
      if event.type == traverse_events['execution'])
  artifacts_ids = set(
      event.artifact_id
      for event in store.get_events_by_execution_ids(executions_ids)
      if event.type == traverse_events['artifact'])
  return [
      artifact for artifact in store.get_artifacts_by_id(artifacts_ids)
      if not filter_type or artifact.type_id == filter_type.id
  ]


def _get_one_hop_executions(
    store: metadata_store.MetadataStore,
    artifact_ids: Iterable[int],
    direction: _Direction,
    filter_type: Optional[metadata_store_pb2.ExecutionType] = None
) -> List[metadata_store_pb2.Execution]:
  """Gets a list of executions within 1-hop neighborhood of the `artifact_ids`.

  Args:
    store: A ml-metadata MetadataStore to look for neighborhood executions.
    artifact_ids: The artifacts' ids in the `store`.
    direction: A direction to specify whether returning ancestors or successors.
    filter_type: An optional type filter of the returned executions, if given
      then only executions of that type is returned.

  Returns:
    A list of qualified executions within 1-hop neighborhood in the `store`.
  """
  if direction == _Direction.ANCESTOR:
    traverse_event = metadata_store_pb2.Event.OUTPUT
  elif direction == _Direction.SUCCESSOR:
    traverse_event = metadata_store_pb2.Event.INPUT
  executions_ids = set(
      event.execution_id
      for event in store.get_events_by_artifact_ids(artifact_ids)
      if event.type == traverse_event)
  return [
      execution for execution in store.get_executions_by_id(executions_ids)
      if not filter_type or execution.type_id == filter_type.id
  ]


def get_metrics_artifacts_for_model(
    store: metadata_store.MetadataStore,
    model_id: int) -> List[metadata_store_pb2.Artifact]:
  """Gets a list of evaluation artifacts from a model artifact.

  It looks for the evaluator component runs that take the given model as input.
  Then it returns the metrics artifact of that component run.

  Args:
    store: A ml-metadata MetadataStore to look for evaluation metrics.
    model_id: The id for the model artifact in the `store`.

  Returns:
    A list of metrics artifacts produced by the Evaluator component runs
    which take the given model artifact as the input.

  Raises:
    ValueError: If the `model_id` cannot be resolved as a model artifact in the
      given `store`.
  """
  pipeline_types = _get_tfx_pipeline_types(store)
  _validate_model_id(store, pipeline_types.model_type, model_id)
  return _get_one_hop_artifacts(store, [model_id], _Direction.SUCCESSOR,
                                pipeline_types.metrics_type)


def get_stats_artifacts_for_model(
    store: metadata_store.MetadataStore,
    model_id: int) -> List[metadata_store_pb2.Artifact]:
  """Gets a list of statistics artifacts from a model artifact.

  It first looks for the input datasets of the trainer that produces the model.
  If the model uses a transformed examples, it further looks for the original
  dataset. Then it returns the statistics artifact of the found dataset(s).

  Args:
    store: A ml-metadata MetadataStore instance.
    model_id: The id for the model artifact in the `store`.

  Returns:
    A list of statistics artifacts produced by the StatsGen component runs
    for the datasets which are used to train the model.

  Raises:
    ValueError: If the `model_id` cannot be resolved as a model artifact in the
      given `store`.
  """
  pipeline_types = _get_tfx_pipeline_types(store)
  _validate_model_id(store, pipeline_types.model_type, model_id)
  trainer_examples = _get_one_hop_artifacts(store, [model_id],
                                            _Direction.ANCESTOR,
                                            pipeline_types.dataset_type)
  # If trainer takes transformed example, we look for its original dataset.
  dataset_ids = set()
  transformed_example_ids = set()
  for example in trainer_examples:
    if example.uri.find('/Transform/'):
      transformed_example_ids.add(example.id)
    else:
      dataset_ids.add(example.id)
  dataset_ids.update(
      dataset.id for dataset in _get_one_hop_artifacts(
          store, transformed_example_ids, _Direction.ANCESTOR,
          pipeline_types.dataset_type))
  return _get_one_hop_artifacts(store, dataset_ids, _Direction.SUCCESSOR,
                                pipeline_types.stats_type)


def _property_value(
    node: Union[metadata_store_pb2.Artifact, metadata_store_pb2.Execution,
                metadata_store_pb2.Context],
    name: Text,
    is_custom_property: bool = False) -> Optional[Union[int, float, Text]]:
  """Given a MLMD node and a (custom) property name, returns its value if any.

  Args:
    node: A node in MLMD lineage graph. It is one of MLMD Artifact, Execution,
      or Context.
    name: The key of the properties or custom properties.
    is_custom_property: Indicates whether the name is a custom property.

  Returns:
    The value of the property if found in the node; If not, returns None.
  """
  properties = node.custom_properties if is_custom_property else node.properties
  if name not in properties:
    return None
  if properties[name].WhichOneof('value') == 'int_value':
    return properties[name].int_value
  if properties[name].WhichOneof('value') == 'float_value':
    return properties[name].double_value
  return properties[name].string_value


def generate_model_card_for_model(store: metadata_store.MetadataStore,
                                  model_id: int) -> ModelCard:
  """Populates model card properties for a model artifact.

  It traverse the parents and children of the model artifact, and maps related
  artifact properties and lineage information to model card property. The
  graphics derived from the artifact payload are handled separately.

  Args:
    store: A ml-metadata MetadataStore instance.
    model_id: The id for the model artifact in the `store`.

  Returns:
    A ModelCard data object with the properties.

  Raises:
    ValueError: If the `model_id` cannot be resolved as a model artifact in the
      given `store`.
  """
  pipeline_types = _get_tfx_pipeline_types(store)
  _validate_model_id(store, pipeline_types.model_type, model_id)
  model_card = ModelCard()
  model_details = model_card.model_details
  trainers = _get_one_hop_executions(store, [model_id], _Direction.ANCESTOR,
                                     pipeline_types.trainer_type)
  if trainers:
    model_details.name = _property_value(trainers[-1], 'module_file')
    model_details.version.name = _property_value(trainers[0], 'checksum_md5')
    model_details.references = [_property_value(trainers[0], 'pipeline_name')]
  stats = get_stats_artifacts_for_model(store, model_id)
  if stats:
    datasets = _get_one_hop_artifacts(store, [stats[-1].id],
                                      _Direction.ANCESTOR,
                                      pipeline_types.dataset_type)
    model_data = model_card.model_parameters.data
    # tfx-oss uses `train` and `eval` splits
    model_data.train.name = os.path.join(datasets[-1].uri, 'train')
    model_data.eval.name = os.path.join(datasets[-1].uri, 'eval')
  return model_card


def read_stats_proto(
    stats_artifact_uri: Text,
    split: Text) -> Optional[statistics_pb2.DatasetFeatureStatisticsList]:
  """Reads DatasetFeatureStatisticsList proto from provided stats artifact uri.

  Args:
    stats_artifact_uri: the output artifact path of a StatsGen component.
    split: the data split to fetch stats from.

  Returns:
    If the artifact uri does not exist, returns None. Otherwise, returns the
    eval split stats as DatasetFeatureStatisticsList.
  """
  stats = statistics_pb2.DatasetFeatureStatisticsList()
  stats_tfrecord_path = os.path.join(stats_artifact_uri,
                                     split, 'stats_tfrecord')
  if not os.path.exists(stats_tfrecord_path):
    logging.warning('No artifact found at %s', stats_tfrecord_path)
    return None
  serialized_stats = next(
      tf.compat.v1.io.tf_record_iterator(stats_tfrecord_path))
  stats.ParseFromString(serialized_stats)
  return stats


def read_metrics_eval_result(
    metrics_artifact_uri: Text) -> Optional[tfma.EvalResult]:
  """Reads TFMA evaluation results from the evaluator output path.

  Args:
    metrics_artifact_uri: the output artifact path of a TFMA component.

  Returns:
    A TFMA EvalResults named tuple including configs and sliced metrics.
    Returns None if no slicing metrics found from `metrics_artifact_uri`.
  """
  result = tfma.load_eval_result(metrics_artifact_uri)
  if not result.slicing_metrics:
    logging.warning('Cannot load eval results from: %s', metrics_artifact_uri)
    return None
  return result



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for model_card_toolkit."""

import json
import os
import uuid

from absl.testing import absltest

from model_card_toolkit import model_card as model_card_module
from model_card_toolkit import model_card_toolkit
from model_card_toolkit.utils.testdata import testdata_utils


class ModelCardToolkitTest(absltest.TestCase):

  def setUp(self):
    super(ModelCardToolkitTest, self).setUp()
    self.tmp_db_path = os.path.join(absltest.get_default_test_tmpdir(),
                                    f'test_mlmd_{uuid.uuid4()}.db')
    self.tmpdir = os.path.join(absltest.get_default_test_tmpdir(),
                               f'model_card_{uuid.uuid4()}')
    if not os.path.exists(self.tmpdir):
      os.makedirs(self.tmpdir)

  def test_init_with_store_no_model_uri(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    with self.assertRaisesRegex(
        ValueError, 'If `mlmd_store` is set, `model_uri` should be set.'):
      model_card_toolkit.ModelCardToolkit(
          output_dir=self.tmpdir, mlmd_store=store)

  def test_init_with_store_model_uri_not_found(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    unknown_model = 'unknown_model'
    with self.assertRaisesRegex(
        ValueError, f'"{unknown_model}" cannot be found in the `mlmd_store`'):
      model_card_toolkit.ModelCardToolkit(
          mlmd_store=store, model_uri=unknown_model)

  def test_scaffold_assets(self):
    output_dir = self.tmpdir
    mct = model_card_toolkit.ModelCardToolkit(output_dir=output_dir)
    self.assertEqual(mct.output_dir, output_dir)
    mc = mct.scaffold_assets()  # pylint: disable=unused-variable
    self.assertIn('default_template.html.jinja',
                  os.listdir(os.path.join(output_dir, 'template/html')))

  def test_scaffold_assets_with_store(self):
    output_dir = self.tmpdir
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    mct = model_card_toolkit.ModelCardToolkit(
        output_dir=output_dir,
        mlmd_store=store,
        model_uri=testdata_utils.TFX_0_21_MODEL_URI)
    mc = mct.scaffold_assets()
    self.assertIsNotNone(mc.model_details.name)
    self.assertIsNotNone(mc.model_details.version.name)
    self.assertNotEmpty(mc.quantitative_analysis.graphics.collection)
    self.assertIn('average_loss', {
        graphic.name for graphic in mc.quantitative_analysis.graphics.collection
    })
    self.assertIn('post_export_metrics/example_count', {
        graphic.name for graphic in mc.quantitative_analysis.graphics.collection
    })
    self.assertIn('default_template.html.jinja',
                  os.listdir(os.path.join(output_dir, 'template/html')))

  def test_update_model_card_with_valid_json(self):
    mct = model_card_toolkit.ModelCardToolkit(output_dir=self.tmpdir)
    valid_model_card = mct.scaffold_assets()
    valid_model_card.schema_version = '0.0.1'
    valid_model_card.model_details.name = 'My Model'
    mct.update_model_card_json(valid_model_card)
    json_path = os.path.join(self.tmpdir, 'data/model_card.json')
    with open(json_path) as f:
      self.assertEqual(
          json.loads(f.read()),
          valid_model_card.to_dict(),
      )

  def test_update_model_card_with_no_version(self):
    mct = model_card_toolkit.ModelCardToolkit()
    model_card_no_version = mct.scaffold_assets()
    model_card_no_version.model_details.name = ('My ' 'Model')
    mct.update_model_card_json(model_card_no_version)
    json_path = os.path.join(mct.output_dir, 'data/model_card.json')
    with open(json_path) as f:
      self.assertEqual(
          json.loads(f.read()),
          model_card_no_version.to_dict(),
      )

  def test_update_model_card_with_invalid_schema_version(self):
    mct = model_card_toolkit.ModelCardToolkit()
    model_card_invalid_version = model_card_module.ModelCard(
        schema_version='100.0.0')
    with self.assertRaisesRegex(ValueError, 'Cannot find schema version'):
      mct.update_model_card_json(model_card_invalid_version)

  def test_export_format(self):
    store = testdata_utils.get_tfx_pipeline_metadata_store(self.tmp_db_path)
    mct = model_card_toolkit.ModelCardToolkit(
        output_dir=self.tmpdir,
        mlmd_store=store,
        model_uri=testdata_utils.TFX_0_21_MODEL_URI)
    model_card = mct.scaffold_assets()
    model_card.schema_version = '0.0.1'
    model_card.model_details.name = 'My Model'
    mct.update_model_card_json(model_card)
    result = mct.export_format()

    model_card_path = os.path.join(self.tmpdir, 'model_cards/model_card.html')
    self.assertTrue(os.path.exists(model_card_path))
    with open(model_card_path) as f:
      content = f.read()
      self.assertEqual(content, result)
      self.assertTrue(content.startswith('<!DOCTYPE html>'))
      self.assertIn('My Model', content)

  def test_export_format_with_customized_template_and_output_name(self):
    mct = model_card_toolkit.ModelCardToolkit(output_dir=self.tmpdir)
    model_card = mct.scaffold_assets()
    model_card.schema_version = '0.0.1'
    model_card.model_details.name = 'My Model'
    mct.update_model_card_json(model_card)

    template_path = os.path.join(self.tmpdir,
                                 'template/html/default_template.html.jinja')
    output_file = 'my_model_card.html'
    result = mct.export_format(
        template_path=template_path, output_file=output_file)

    model_card_path = os.path.join(self.tmpdir, 'model_cards', output_file)
    self.assertTrue(os.path.exists(model_card_path))
    with open(model_card_path) as f:
      content = f.read()
      self.assertEqual(content, result)
      self.assertTrue(content.startswith('<!DOCTYPE html>'))
      self.assertIn('My Model', content)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Setup to install the Model Card Toolkit."""

from setuptools import find_packages
from setuptools import setup

REQUIRED_PACKAGES = [
    'absl-py>=0.7,<0.9',
    'semantic-version>=2.8.0,<3',
    'jinja2>=2.10,<3',
    'matplotlib>=3.2.0,<4',
    'jsonschema>=3.2.0,<4',
    'tensorflow-data-validation>=0.21.0,<0.23',
    'tensorflow-model-analysis>=0.21.0,<0.23',
    'tensorflow-metadata>=0.21.0,<0.23',
    'ml-metadata>=0.21.0,<0.23',
]

# Get version from version module.
with open('model_card_toolkit/version.py') as fp:
  globals_dict = {}
  exec(fp.read(), globals_dict)  # pylint: disable=exec-used
__version__ = globals_dict['__version__']

with open('README.md', 'r', encoding='utf-8') as fh:
  _LONG_DESCRIPTION = fh.read()

setup(
    name='model-card-toolkit',
    version=__version__,
    description='Model Card Toolkit',
    long_description=_LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/tensorflow/model-card-toolkit',
    author='Google LLC',
    author_email='tensorflow-extended-dev@googlegroups.com',
    packages=find_packages(),
    package_data={
        'model_card_toolkit': [
            'schema/**/*.json', 'template/**/*.jinja',
            'template/**/*.jinja.test'
        ]
    },
    python_requires='>=3.5,<4',
    install_requires=REQUIRED_PACKAGES,
    tests_require=REQUIRED_PACKAGES,
    # PyPI package information.
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    license='Apache 2.0',
    keywords='model card toolkit ml metadata machine learning',
)
